package com.caimei.mapper;

import com.caimei.model.po.*;
import com.caimei.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/26
 */
@Mapper
public interface PayOrderMapper {
    /**
     * 查询订单信息
     *
     * @param orderId
     * @return
     */
    OrderVo findOrder(Integer orderId);

    /**
     * 查询支付记录
     *
     * @param order
     * @return
     */
    List<DiscernReceiptVo> getDiscernReceipt(OrderVo order);

    /**
     * 更新订单信息
     *
     * @param order
     */
    void updateSelective(OrderVo order);

    /**
     * 保存收款记录
     *
     * @param discernReceipt
     */
    void insertDiscernReceipt(CmDiscernReceiptPo discernReceipt);

    /**
     * 保存订单收款关系
     *
     * @param relation
     */
    void insertOrderRelation(CmReceiptOrderRelationPo relation);

    /**
     * 查询线上支付链接
     *
     * @param orderId
     * @param amount
     * @return
     */
    OrderPayLinkVo findOrderPayLink(@Param("orderId") Integer orderId, @Param("amount") BigDecimal amount);

    /**
     * 更新线上支付链接状态
     *
     * @param orderPayLink
     */
    void updateOrderPayLink(OrderPayLinkVo orderPayLink);

    /**
     * 保存线上支付链接
     *
     * @param orderPayLink
     */
    void insertOrderPayLink(OrderPayLinkVo orderPayLink);

    /**
     * 查询线上支付链接
     *
     * @param linkLogo
     * @return
     */
    OrderPayLinkVo getOrderPayLink(String linkLogo);

    /**
     * 查询用户信息
     *
     * @param userId
     * @return
     */
    UserPo findUser(Long userId);

    /**
     * 查询订单商品
     *
     * @param orderId
     * @return
     */
    List<OrderProductVo> findAllOrderProduct(Integer orderId);

    /**
     * 保存分账信息
     *
     * @param splitAccount
     */
    void insertSplitAccount(SplitAccountVo splitAccount);

    /**
     * 获取线上支付开关状态
     */
    Integer getPayOnLineSwitch();

    /**
     * 查询子商户商编
     *
     * @param shopId
     * @return
     */
    String findCommercialCode(Long shopId);

    /**
     * 查询已支付金额
     *
     * @param orderProductId
     * @return
     */
    BigDecimal findPaidAmount(Integer orderProductId);

    /**
     * 查询子订单
     *
     * @param orderId
     * @return
     */
    List<ShopOrderVo> findShopOrder(Integer orderId);

    /**
     * 查询已支付运费
     *
     * @param orderId
     * @param shopId
     * @return
     */
    BigDecimal findShipping(@Param("orderId") Integer orderId, @Param("shopId") Integer shopId);

    /**
     * 修改子订单付款状态及付款金额
     *
     * @param shopOrderId 子订单id
     * @param paidShop    已支付金额
     * @param payStatus   付款状态
     */
    void updateShopOrderByPayStatus(@Param("shopOrderId") Integer shopOrderId, @Param("paidShop") BigDecimal paidShop, @Param("payStatus") String payStatus);

    /**
     * 修改主订单付款状态
     *
     * @param orderId
     * @param payStatus
     */
    void updateOrderByPayStatus(@Param("orderId") Integer orderId, @Param("payStatus") String payStatus);
}
