package com.caimei.service.impl;

import com.caimei.mapper.OrderMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.enumerate.ReceivablesType;
import com.caimei.model.po.PromotionsPo;
import com.caimei.model.vo.*;
import com.caimei.service.OrderService;
import com.caimei.util.MathUtil;
import com.caimei.util.ProductUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/26
 */
@Service
public class OrderServiceImpl implements OrderService {
    @Resource
    private OrderMapper orderMapper;
    @Value("${caimei.oldapi}")
    private String domain;

    @Override
    public ResponseJson<PageInfo<OrderVo>> getOrderList(Integer orderState, Integer userId, Integer organizeId, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<OrderVo> orderList = orderMapper.findOrderList(userId, orderState, organizeId);
        for (OrderVo order : orderList) {
            //111,待付待收待发
            if ("11".equals(order.getStatus()) && "1".equals(order.getPayStatus())) {
                order.setStatus("111");
            }
            //判断交易全退情况下,是否发过货,77,交易全退可以查看物流
            List<LogisticsBatchVo> batchList = orderMapper.findLogistics(order.getOrderId());
            if ("7".equals(order.getStatus()) && batchList != null && batchList.size() > 0) {
                order.setStatus("77");
            }
            List<ShopOrderVo> shopOrderList = orderMapper.findAllShopOrder(order.getOrderId());
            for (ShopOrderVo shopOrder : shopOrderList) {
                List<OrderProductVo> orderProductList = orderMapper.findOrderProduct(shopOrder.getShopOrderId());
                for (OrderProductVo orderProduct : orderProductList) {
                    orderProduct.setProductImage(ProductUtils.getImageURL("product", orderProduct.getProductImage(), 0, domain));
                    //不含税可开票商品,单价/折后单价=税前单价+税费
                    if ("0".equals(orderProduct.getIncludedTax()) && ("1".equals(orderProduct.getInvoiceType()) || "2".equals(orderProduct.getInvoiceType()))) {
                        BigDecimal valueTax = MathUtil.div(MathUtil.mul(orderProduct.getPrice(), orderProduct.getTaxRate()), 100);
                        orderProduct.setPrice(MathUtil.add(orderProduct.getPrice(), valueTax));
                        orderProduct.setDiscountPrice(MathUtil.add(orderProduct.getPrice(), orderProduct.getAddedValueTax()));
                    }
                    //查询订单下商品的促销活动
                    if (orderProduct.getOrderPromotionsId() != null && orderProduct.getOrderPromotionsId() > 0) {
                        PromotionsPo promotions = orderMapper.findOrderPromotionsById(orderProduct.getOrderPromotionsId());
                        orderProduct.setProductPromotion(promotions);
                        if (promotions != null && promotions.getType() == 1 && promotions.getMode() == 1) {
                            if ("0".equals(orderProduct.getIncludedTax()) && ("1".equals(orderProduct.getInvoiceType()) || "2".equals(orderProduct.getInvoiceType()))) {
                                promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), orderProduct.getAddedValueTax()));
                            }
                        }
                    }
                }
                shopOrder.setOrderProductList(orderProductList);
                shopOrder.setShopLogo(ProductUtils.getImageURL("shopLogo", shopOrder.getShopLogo(), 0, domain));
            }
            //过滤运费商品
            shopOrderList.removeIf(shopOrderVo -> shopOrderVo.getShopID() == 998);
            order.setShopOrderList(shopOrderList);
            //设置付款金额
            getDiscernReceipt(order);
        }
        PageInfo<OrderVo> pageData = new PageInfo<>(orderList);
        return ResponseJson.success(pageData);
    }

    /**
     * 支付金额,待付金额
     */
    public List<DiscernReceiptVo> getDiscernReceipt(OrderVo order) {
        //支付记录
        List<DiscernReceiptVo> discernReceiptList = orderMapper.findDiscernReceipt(order.getOrderId());
        BigDecimal receiptAmount = BigDecimal.ZERO;
        //订单款
        if (discernReceiptList != null && discernReceiptList.size() > 0) {
            for (DiscernReceiptVo discernReceipt : discernReceiptList) {
                receiptAmount = receiptAmount.add(discernReceipt.getAssociateAmount());
                if (discernReceipt.getPayType() != null) {
                    discernReceipt.setPayType(ReceivablesType.getReceivablesType(Integer.parseInt(discernReceipt.getPayType())));
                }
            }
        }
        //判断是否可以走线上支付
        DiscernReceiptVo discernReceipt = orderMapper.findOfflinePayment(order.getOrderId());
        if (null == discernReceipt) {
            order.setOnlinePayFlag("0");
        } else {
            order.setOnlinePayFlag("1");
        }
        //付供应商总金额 + 默认手续费 > 订单总金额
        BigDecimal payTotalFee = order.getPayTotalFee();
        BigDecimal handlingFee = MathUtil.mul(payTotalFee, 0.0038, 2);
        if (MathUtil.compare(handlingFee, 8) < 0) {
            handlingFee = new BigDecimal(8);
        }
        List<ShopOrderVo> shopOrderList = orderMapper.findAllShopOrder(order.getOrderId());
        for (ShopOrderVo shopOrder : shopOrderList) {
            handlingFee = MathUtil.add(handlingFee, shopOrder.getShouldPayShopAmount());
        }
        if (MathUtil.compare(payTotalFee, handlingFee) < 0) {
            order.setPayButton(true);
        }
        //待付总金额
        order.setPendingPayments(MathUtil.sub(order.getPayableAmount(), receiptAmount));
        //支付总金额
        order.setReceiptAmount(receiptAmount);
        return discernReceiptList;
    }
}

